-- Position and size of the camera view
local offset_x = 10
local offset_y = 80
local cam_view_x, cam_view_y = 1300, 800

-- Position of the editor window
local editor_window_offset_X = 0
local editor_window_offset_Y = 0

-- Initial scale
local zoom_factor = 1.0

local world = {
    width   = 5000,
    height  = 5000,
    rows    = 100,
    columns = 100
  }
  
local cursor = {
    x       = world.width/2+editor_window_offset_X+offset_x,
    y       = world.height/2+editor_window_offset_Y+offset_y,
    --x         = 1,
    --y         = 1,
    width   = 1,
    height  = 1,
    speed   = 700
}

function zoom(direction)
  if direction == '+' then
    if zoom_factor > 15 then
      zoom_factor = 15
    else
      zoom_factor = zoom_factor + 0.05
    end
  elseif direction == '-' then
    if zoom_factor < 0.1 then
      zoom_factor = 0.1
    else
      zoom_factor = zoom_factor - 0.05
    end
  end
  cam:setScale(zoom_factor)
end

function moveCursor(dt)
  -- this is to move the camera as the camera follows the cursor
  local dx = love.keyboard.isDown('a') and -1 or (love.keyboard.isDown('d') and 1 or 0)
  local dy = love.keyboard.isDown('w') and -1 or (love.keyboard.isDown('s') and 1 or 0)
  if cursor.x < offset_x then
    cursor.x = offset_x+1
  elseif cursor.x > world.width+offset_x then
    cursor.x = world.width+offset_x-1
  elseif cursor.y < offset_y then
    cursor.y = offset_y+1
  elseif cursor.y > world.height+offset_y then
    cursor.y = world.width+offset_y-1
  else
    cursor.x = cursor.x + dx * dt * cursor.speed
    cursor.y = cursor.y + dy * dt * cursor.speed
  end
end


function EditorSetup()
  cam = gamera.new(offset_x,offset_y,world.width,world.height)
  cam:setWindow(offset_x,offset_y,cam_view_x,cam_view_y)
end


local function drawCursor()
  love.graphics.rectangle('line',
                          cursor.x - cursor.width / 2,
                          cursor.y - cursor.height / 2,
                          cursor.width,
                          cursor.height)
end

local function drawGrid()
  local columnWidth = world.width / world.columns
  local rowHeight = world.height / world.rows
  
  -- Set color for the grid lines
  love.graphics.setColor(Color('cdarkgrey'))
  
  -- draw exterior lines
  -- top line left to right
  --love.graphics.line(offset_x+editor_window_offset_X, offset_y+editor_window_offset_Y, offset_x+editor_window_offset_X, world.height+editor_window_offset_Y)
  -- bottom line left to right
  --love.graphics.line(offset_x+editor_window_offset_X, world.height+editor_window_offset_Y, world.width+editor_window_offset_X, world.height+editor_window_offset_Y)
  -- left line top to bottom
  --love.graphics.line(offset_x+editor_window_offset_X, offset_y+editor_window_offset_Y, offset_x+editor_window_offset_X, world.height+editor_window_offset_Y)
  -- right line top to bottom
  --love.graphics.line(world.width+editor_window_offset_X, offset_y+editor_window_offset_Y, world.width+editor_window_offset_X, world.height+editor_window_offset_Y)
  
  -- draw column-lines
  for x=1, world.columns do
    love.graphics.line((x*columnWidth)+offset_x+editor_window_offset_X, offset_y+editor_window_offset_Y, (x*columnWidth)+offset_x+editor_window_offset_X, world.height+offset_y+editor_window_offset_Y)
  end
  
  -- draw row lines
  for y=1, world.rows do
    love.graphics.line(offset_x+editor_window_offset_X, (y*rowHeight)+offset_y+editor_window_offset_Y, world.width+offset_y+editor_window_offset_X, (y*rowHeight)+offset_y+editor_window_offset_Y)
  end
  
  -- draw cell numbers
  
  local cell = 1
  for y=1, world.rows do
    for x=1, world.columns do
      love.graphics.print(cell, (x*columnWidth)-40+offset_x+editor_window_offset_X, (y*rowHeight)-30+offset_y+editor_window_offset_Y)
      cell = cell+1
    end
  end
  --]]
end

function EditorDraw()
  -- set position of camera whereever the cursor is
  cam:setPosition(cursor.x, cursor.y)
  
  -- then draw camera-dependent stuff, including the cursor
  cam:draw(function(l,t,w,h)
    drawCursor()
    drawGrid()
  end)

  -- this is outside of camera.draw so it is not affected by the scroll
  local m, n = love.mouse.getPosition()
  local MouseX, MouseY = push:toGame(m, n)
  if MouseX == nil then
    MouseX = 0
  end
  if MouseY == nil then
    MouseY = 0
  end
  
  love.graphics.setColor(Color('white'))
  love.graphics.print("Mouse position: " .. m .. " " .. n, 1350, 420)
  love.graphics.print("Push Mouse position: " .. MouseX .. " " .. MouseY, 1350, 450)
  love.graphics.print("Hover: " .. hoverField, 1350, 470)
  love.graphics.print("Fielder: " .. xFielder, 1350, 490)
  
  local camera_x, camera_y = cam:getPosition()
  local camera_l, camera_t, camera_w, camera_h = cam:getVisible()
  love.graphics.print("Camera Position: X: " .. camera_x .. " Y: " .. camera_y, 1350, 510)
  love.graphics.print("Camera Visible: L: " .. camera_l .. " T: " .. camera_t .. " W: " .. camera_w .. " H: " .. camera_h, 1350, 530)
  love.graphics.print("Player Position: X: " .. cursor.x .. " Y: " .. cursor.y, 1350, 550)
  
  local fps = love.timer.getFPS()
  love.graphics.print("FPS: " .. fps, 1350, 570)

end

function EditorUpdate(dt)
  -- Update drawing depending on cursor movement or zooming
  moveCursor(dt)
  zoom()
  
  -- determine grid field depending on mouse hover (taking into account scaling and camera position)
  -- account for scaling
  local m, n = love.mouse.getPosition()
  local MouseX, MouseY = push:toGame(m, n)
 
  if MouseX == nil then
    MouseX = 0
  end
  if MouseY == nil then
    MouseY = 0
  end
 
  -- account for editor window position
  MouseX = MouseX - editor_window_offset_X - offset_x
  MouseY = MouseY - editor_window_offset_Y - offset_y
 
  -- account for camera position and zoom factor
  local camera_l, camera_t, camera_w, camera_h = cam:getVisible() 
 
  if zoom_factor ~= 1.0 then
    MouseX = (MouseX/zoom_factor)+camera_l
    MouseY = (MouseY/zoom_factor)+camera_t
  else
    MouseX = MouseX+camera_l
    MouseY = MouseY+camera_t
  end

    
  -- Calculate row and column the mouse hovers over
  local columnWidth = world.width / world.columns
  local rowHeight = world.height / world.rows
  local col = math.ceil((MouseX-offset_x)/columnWidth)
  local row = math.ceil((MouseY-offset_y)/rowHeight)
  hoverField = ((row-1)*world.columns)+col 
  
  -- calculate neighbor fields
  xFielder = hoverField .. " " .. (hoverField + 1) .. " " .. (hoverField+world.columns) .. " " .. (hoverField+world.columns+1)
end
